/* 
 *  getstatd - User statistic visualization daemon.
 *
 *  Copyright (c) 1998  Maxim Chirkov <mc@skyway.ru>  
 */

#ifndef __GETSTAT__
#define __GETSTAT__

#include "config.h"
#include "cfg_mgr2.h"

#ifdef HAVE_UNISTD_H
#	include <unistd.h>
#endif

#ifdef HAVE_FCNTL_H
#	include <fcntl.h>
#endif

#ifdef HAVE_LIMITS_H
#	include <limits.h>
#endif

#ifdef HAVE_PATHS_H
#	include <paths.h>
#endif

#ifdef HAVE_SYSLOG_H
#	include <syslog.h>
#endif

#ifdef HAVE_STDARG_H
#	include <stdarg.h>
#else
#	ifdef HAVE_VARARGS_H
#		include <varargs.h>
#	endif
#endif

#ifdef HAVE_LOCALE_H
#	include <locale.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>
#include <utmp.h>
#include <signal.h>
#include <string.h>

/* #include <netinet/in.h> */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

/*
#include <net/if.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
*/

#ifndef NOSSL
#	include <ssl.h>
#	include <pem.h>
#endif


#ifndef EOF
#	define EOF (-1)
#endif
#ifndef NULL
#	if !defined(__cplusplus) || defined(__GNUC__)
#		define NULL ((void*)0)
#	else
#		define NULL (0)
#	endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef HAVE_BCOPY
#	define bcopy(s,d,l) memcpy((d),(s),(l))
#endif

/*
 * Types.
 */

typedef void (*form_parser)(void);
typedef void (*form_parser_s)(char *, int);
typedef void (*form_sub_parser)(char *,char *, int);


typedef	void	sig_func(int);	


typedef struct _FORM_ITEM_ {
  char * name;
  form_parser  parser;
} form_item;

typedef struct _FORM_ITEM_S_ {
  char * name;
  form_parser_s  parser;
} form_item_s;

typedef struct _FORM_SUB_ITEM_ {
  char * name;
  form_sub_parser  parser;
} form_sub_item;

typedef char * mime_types;

typedef struct _TIME_PERIOD_ {
    char * name;
    int   base_hour;
    int	  period;
    long  all_time;
    float  price;
    struct _TIME_PERIOD_ * next;
} time_period;

#define PN_SIZE 16
typedef struct _SIMPLE_TIME_PERIOD_ {
    char  name[PN_SIZE];
    long  all_time;
    float price;
} simple_time_period;

typedef struct _USER_LIST_ {
    char	* user_name;
    long	all_count;
    long	all_time;
    simple_time_period *tp;
    struct _USER_LIST_ * next;
} user_list;


typedef struct _TTY_LIST_ {
    char	* tty_name;
    long	all_count;
    long	all_time;
    simple_time_period *tp;
    time_t	begin_time;
    time_t	end_time;
    struct _TTY_LIST_ * next;
} tty_list;


typedef struct _IGNORE_LIST_ {
    char	* user_name;
    struct _IGNORE_LIST_ * next;
} ignore_list;

#if defined(XTACACS4) || defined(XTACACS3)
    
#	undef HAVE_ADDR_IN_UTMP
#	undef UT_LINESIZE
#	undef UT_NAMESIZE
#	undef UT_HOSTSIZE
#	define UT_LINESIZE	8
#	define UT_NAMESIZE	8
#	define UT_HOSTSIZE	16
    
#	ifdef XTACACS3
	typedef struct tacacs3_utmp {
          char ut_line[8];
          char ut_name[8];
          char ut_host[16];
          time_t ut_time;
	} def_utmp;
#	endif

#	ifdef XTACACS4
	typedef struct tacacs4_utmp {
          char ut_line[8];
          char ut_user[8];
          char ut_host[16];
          time_t ut_time;
          char ut_comment[16];
	} def_utmp;
#	endif
#else 
    typedef struct utmp def_utmp;
#endif


typedef struct _UTMP_LIST_ {
    def_utmp ut;
    struct _UTMP_LIST_ *next;
    struct _UTMP_LIST_ *prev;
} utmp_list;


typedef struct _P_HOSTS_ALLOW_ {
    unsigned long int netaddr;
    unsigned long int netmask;
    int flag_admin;
    int flag_plugin;
    int flag_users;
    int flag_deny;
    int flag_close;
    int flag_all;
    struct _P_HOSTS_ALLOW_ * next;
} p_hosts_allow;


typedef struct _FORM_TREE_ {
    char * name;
    int	mime_type;
    char * form;
    struct _FORM_TREE_ * next;
} form_tree;


typedef struct _PLUGIN_LIST_ {
    char * name;
    char * iostdin;
    char * proc;
    char * header;
    struct _PLUGIN_LIST_ * next;
} plugin_list;


/*
 *  Definitions.
 */

/* Base defaults */

#ifndef DAEMON_UID
#	define DAEMON_UID		65534 
#endif
#	define EVENTR			"<TR BGCOLOR=\"#EDBE82\">"
#	define NONEVENTR		"<TR BGCOLOR=\"#FFCC99\">"
#	define ROOT_HTTP_PORT		80
#	define REFRESH_FORM		99999
/* Addition parameters */
    
#	define BUF_SIZE			256
#	define TIME_SIZE 		64
#	define TERM_SIZE 		128

#ifndef UT_LINESIZE
#	define UT_LINESIZE		8
#endif

#ifndef UT_NAMESIZE
#	define UT_NAMESIZE		8
#endif

#define _HOST_AUTH		"localhost"
#define POP3_PORT		110
#define DEFAULT_PORT		1080
#define DEFAULT_NIGHTTIME   	7
#define DEFAULT_NIGHTFROM   	0
#define _DEF_WTMP		"/var/adm/wtmp"
#define _DEF_UTMP		"/etc/utmp"

#define OK_ANS 			"+OK"
#define OK_ANS_SIZE 		3
#define USER_QUERY		"USER"
#define PASS_QUERY		"PASS"
#define QUIT_QUERY		"QUIT"
#define DILIMERS 		", \t\n\r:;/\\#"
#define ALARM_TIME 		60

#define MIME_NONE		0
#define MIME_HTML		1
#define MIME_PLAIN		2
#define MIME_GIF		3
#define MIME_JPEG		4
#define MIME_PHTML		5

#define UT_MAIN 		0 /* Index request */
#define UT_USER 		1 /* Current user statistic */
#define UT_ADMIN 		2 /* Admin statistic */
#define UT_PLUGIN 		3 /* Plugin statistic */

#define CMD_ZERO		0 /* Null operation */
#define CMD_USERSTAT	1 /* Perform current user statistic */
#define CMD_LASTSTAT	2 /* Perform last users statistic */
#define CMD_LINESTAT	3 /* Perform line statistic */
#define CMD_ALLUSERSTAT	4 /* Perform all users statistic */

#define R_CRASH		1 /* No logout record, system boot in between */
#define R_DOWN		2 /* System brought down in decent way */
#define R_NORMAL		3 /* Normal */
#define R_NOW		4 /* Still logged in */
#define R_REBOOT		5 /* Reboot record. */

/*
 *  Global variables.
 */

extern char 		* conf_path;
extern long 		serv_port;
extern long 		low_serv_port;
extern str_array 	* ipallow;
extern int 		n_ipallow;
extern str_array	* calctty;
extern int 		n_calctty;
extern char 		* wtmp_path;
extern char 		* utmp_path;
extern char 		* index_path;
extern char 		* deny_path;
extern char 		* host_auth;
extern int		gen_socket;
extern long		pop3_port;
extern char 		* deny_identification;
extern char 		* deny_passwd;
extern char 		* deny_right;
extern char 		* userlist;
extern char 		* adminlist;
extern char 		* pluginlist;
extern char 		* ignoreusers;
extern char 		* pwdlist;
extern int 		pwdcrypt;
extern int    		logadmins;
extern int    		logplugins;
extern int    		logusers;
extern int    		logipdeny;
extern char 		* autoauthdir;
extern char 		* plugin_index;
extern char 		* user_index;
extern char 		* admin_index;
extern long		refresh;
extern str_array	* hostsallow;
extern int 		n_hostsallow;
extern str_array	* timeperiods;
extern int 		n_timeperiods;
extern p_hosts_allow    * post_hostsallow;
extern time_period      * post_timeperiods;
extern int 		invertignoreusers;
extern int 		invertpluginlist;
extern int 		invertadminlist;
extern int 		invertuserlist;
extern int    		use_ssl;
extern char 		* ssl_key;
extern char 		* ssl_certificate;
extern char 		* wtmp_arc_mask;
extern char 		* wtmp_arc_expand;
extern double 		permanent_price;
extern double		minimum_price;
extern plugin_list      * post_plugins;
extern str_array	* plugins;
extern int 		n_plugins;

extern time_t 		start_t;
extern time_t 		end_t;
extern long		all_count;
extern long		all_night;
extern long 		all_count;
extern long 		line_count;
extern ignore_list 	* ignore_userlist;
extern user_list 	* all_userlist;
extern tty_list		* all_ttylist;
extern long 		* coop_tty;
extern char 		user_name[BUF_SIZE+1];
extern char 		cur_ip[BUF_SIZE+1];
extern char 		gen_ipaddr[BUF_SIZE+1];
simple_time_period 	* def_tp;
extern int		speed_flag;

#ifndef NOSSL
extern SSL 		* ssl_socket;
extern SSL_CTX 		* ssl_ctx;
extern X509 		* public_cert;
extern RSA 		* private_key;
#endif
#ifndef HAVE_SNPRINTF
	extern int snprintf (char *str, size_t count, const char *fmt, ...);
#endif /* HAVE_SNPRINTF */


/* Useful functions */

extern int readline(int fd, char *ptr, int maxlen);
extern int writeline(int fd, char *ptr);
extern void prints( char *ptr);
extern void log_info(char *str);
extern void log_error(char *str);
extern char * uudecode( char *bufcoded);
extern void fill_null( char * str);
extern void mark_first_word(char *item);
extern time_t make_utc_time (time_t user_time, int night_time, int per_size);
extern void log_user_access(int cmd_flag);
extern void log_ip_deny(unsigned long int addr);
extern int check_tty(char * ut_line);
extern int check_host(unsigned long int addr, int service_flag);
extern void * my_malloc(size_t size);
extern void  my_free(void *ptr);
extern int prep_period ( time_period * tp, char * period_name, long * all_time, float * price);


/* Signals */
extern void ignore_trace (int sig);
extern void kill_trace (int sig);
extern void alarm_trace (int sig);
extern void chld_trace (int sig);
extern void hup_trace (int sig);
extern void segv_trace (int sig);
extern void set_signal(int sig_num, sig_func * func);


/* Identification */
extern char * get_ip_user(char *ip_name);
extern char * make_user_name (struct sockaddr_in cli_addr);
extern char * get_tty_user(char *tty_name,char * user_name);
extern int pwdcheck (int isauth);
extern void get_auth();


/* Process query */
extern void prog_req(int socket,struct sockaddr_in	cli_addr);
extern int trans_query (time_t *start_date, time_t *end_date, char * cmd);
extern int get_query_item ( char * query, char * item, char * result, int result_size);

/* SSL function */
#ifndef NOSSL
extern void ssl_setup(int sockfd);
extern void ssl_init();
#endif

/* Statistic generation */
extern int print_time_stat(time_t start_date, time_t end_date, char * user_name, int cmd_flag);
extern void prepare_lists(void);


/* Process configuration items */
extern void load_config ( void );
extern void reload_config ( void );
extern void prepare_hostsallow( void );
extern void prepare_timeperiod( void );
extern void prepare_plugins( void );


/* HTML includes */
extern void get_file (int socket, char * path_name, int mime_flag);
extern int perform_includes(char * buff, int socket, char * string, int str_size );

extern void inc_who ( void );
extern void inc_line ( void );
extern void inc_last ( void );
extern void inc_users ( void );
extern void inc_statususer ( char * string, int str_size );
extern void inc_fromdate ( char * string, int str_size);
extern void inc_todate ( char * string, int str_size );
extern void inc_statistic ( void );
extern void inc_sumhour ( char * string, int str_size );
extern void inc_summin ( char * string, int str_size );
extern void inc_tp_sumhour ( char * period_name, char * string, int str_size );
extern void inc_tp_summin ( char * period_name, char * string, int str_size );
extern void inc_tp_rsumhour ( char * period_name, char * string, int str_size );
extern void inc_tp_rsummin ( char * period_name, char * string, int str_size );
extern void inc_tp_price ( char * period_name,char * string, int str_size );
extern void inc_tp_sumprice (  char * string, int str_size  );
extern void inc_tp_real_sumprice ( char * string, int str_size );
extern void inc_version ( void );
extern void inc_refresh ( void );
extern void inc_s_longdate ( char * string, int str_size );
extern void inc_e_longdate ( char * string, int str_size );
extern void inc_s_month ( char * string, int str_size );
extern void inc_s_day ( char * string, int str_size );
extern void inc_s_hour ( char * string, int str_size );
extern void inc_s_year ( char * string, int str_size );
extern void inc_e_month ( char * string, int str_size );
extern void inc_e_day ( char * string, int str_size );
extern void inc_e_hour ( char * string, int str_size );
extern void inc_e_year ( char * string, int str_size );
extern void inc_pluginheader ( char * name, char * string, int str_size );
extern void inc_plugin ( char * name,char * string, int str_size );
#ifdef __cplusplus
}
#endif
#endif