/* 
 *  loadcfg.c - Processing configuration items.
 *
 *  Copyright (c) 1998  Maxim Chirkov <mc@skyway.ru>  
 */

#include	"getstatd.h"
#include 	"cfg_mgr2.h"

long	low_serv_port = 0;
long serv_port = DEFAULT_PORT;
str_array * ipallow = NULL;
int n_ipallow = 0;
str_array * calctty = NULL;
int n_calctty = 0;
char * wtmp_path = NULL;
char * utmp_path = NULL;
char * index_path = NULL;
char * deny_path = NULL;
long pop3_port = POP3_PORT;
char * host_auth = NULL;
char * deny_identification = NULL;
char * deny_passwd = NULL;
char * deny_right = NULL;
char * autoauthdir = NULL;
char * userlist = NULL;
char * adminlist = NULL;
char * pluginlist = NULL;
char * ignoreusers = NULL;
char * pwdlist = NULL;
char * plugin_index = NULL;
char * user_index = NULL;
char * admin_index = NULL;
int    pwdcrypt=1;
int    logadmins=0;
int    logplugins=0;
int    logusers=0;
int    logipdeny=0;
long	refresh = REFRESH_FORM;
str_array	* hostsallow = NULL;
int 	n_hostsallow = 0;
str_array	* timeperiods = NULL;
int 	n_timeperiods = 0;
int invertignoreusers = 0;
int invertpluginlist = 0;
int invertadminlist = 0;
int invertuserlist = 0;
int    use_ssl = 0;
char * ssl_key = NULL;
char * ssl_certificate = NULL;
char * wtmp_arc_mask = NULL;
char * wtmp_arc_expand = NULL;
double permanent_price=0;
double minimum_price=0;
str_array	* plugins = NULL;
int 	n_plugins = 0;

cfg_item items[] = {{"port","",0,&serv_port,0,p_uint},
                    {"calctty","",0,&calctty,&n_calctty,p_arrstring},
                    {"ipallow","",0,&ipallow,&n_ipallow,p_arrstring},
                    {"wtmp","",0,&wtmp_path,0,p_string},
                    {"utmp","",0,&utmp_path,0,p_string},
                    {"index","",0,&index_path,0,p_string},
                    {"deny","",0,&deny_path,0,p_string},
                    {"portauth","",0,&pop3_port,0,p_uint},
                    {"hostauth","",0,&host_auth,0,p_string},
                    {"deny_identification","",0,&deny_identification,0,p_string},
                    {"deny_passwd","",0,&deny_passwd,0,p_string},
                    {"deny_right","",0,&deny_right,0,p_string},
                    {"autoauthdir","",0,&autoauthdir,0,p_string},
                    {"userlist","",0,&userlist,0,p_string},
                    {"adminlist","",0,&adminlist,0,p_string},
                    {"pluginlist","",0,&pluginlist,0,p_string},
                    {"pwdlist","",0,&pwdlist,0,p_string},
                    {"pwdcrypt","",0,&pwdcrypt,0,p_bool},
                    {"logadmins","",0,&logadmins,0,p_bool},
                    {"logplugins","",0,&logplugins,0,p_bool},
                    {"logusers","",0,&logusers,0,p_bool},
                    {"logipdeny","",0,&logipdeny,0,p_bool},
                    {"plugin_index","",0,&plugin_index,0,p_string},
                    {"user_index","",0,&user_index,0,p_string},
                    {"admin_index","",0,&admin_index,0,p_string},
                    {"ignoreusers","",0,&ignoreusers,0,p_string},
                    {"refresh","",0,&refresh,0,p_long},
                    {"hostsallow","",0,&hostsallow,&n_hostsallow,p_arrstring},
                    {"timeperiods","",0,&timeperiods,&n_timeperiods,p_arrstring},
                    {"invertignoreusers","",0,&invertignoreusers,0,p_bool},
                    {"invertpluginlist","",0,&invertpluginlist,0,p_bool},
                    {"invertadminlist","",0,&invertadminlist,0,p_bool},
                    {"invertuserlist","",0,&invertuserlist,0,p_bool},
                    {"use_ssl","",0,&use_ssl,0,p_bool},
                    {"ssl_key","",0,&ssl_key,0,p_string},
                    {"ssl_certificate","",0,&ssl_certificate,0,p_string},
                    {"wtmp_arc_mask","",0,&wtmp_arc_mask,0,p_string},
                    {"wtmp_arc_expand","",0,&wtmp_arc_expand,0,p_string},
                    {"permanent_price","",0,&permanent_price,0,p_double},
                    {"minimum_price","",0,&minimum_price,0,p_double},
                    {"plugins","",0,&plugins,&n_plugins,p_arrstring},
		    {0,0,0,0,0}};

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int check_open(char * file_name, int flag){
    char * buff;
    
  if ((file_name == NULL) || ((access(file_name, R_OK)) != 0)) {
	buff = (char *) my_malloc(BUF_SIZE+1);
        snprintf (buff, BUF_SIZE, "can't open file [%s]. Check configuration file.",file_name);
	if (flag == 0){	
	    log_error(buff);
	} else {
	    log_info(buff);
	    my_free(buff);
	    return -1;
	}
  }
  return 0;
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void load_config ( void ){
        struct hostent *hp;

    cfg_item_process( conf_path, items);

    if (n_ipallow == 0){
        log_info("No 'ipallow' rules found.");
    }

    if (n_calctty == 0){
        log_error("No 'calctty' rules found.");
    }

    if (n_hostsallow == 0){
        log_error("No 'hostsallow' rules found.");
    }
    prepare_hostsallow();

    if (n_timeperiods == 0){
        log_info("No 'timeperiods' rules found.");
    }
    prepare_timeperiod();
    if (n_plugins == 0){
        log_info("No 'plugins' rules found.");
    }
    prepare_plugins();

    if (wtmp_path == NULL){
#ifdef _PATH_WTMP
        wtmp_path= (char *) my_malloc (strlen(_PATH_WTMP));
	strcpy(wtmp_path,_PATH_WTMP);
#else
        wtmp_path= (char *) my_malloc (strlen(_DEF_WTMP));
        strcpy(wtmp_path,_DEF_WTMP);
#endif
    }

    if (utmp_path == NULL){
#ifdef _PATH_UTMP
        utmp_path= (char *) my_malloc (strlen(_PATH_UTMP));
	strcpy(utmp_path,_PATH_UTMP);
#else
        utmp_path= (char *) my_malloc (strlen(_DEF_UTMP));
        strcpy(utmp_path,_DEF_UTMP);
#endif
    }

    if (host_auth == NULL){
        host_auth = (char *) my_malloc (strlen(_HOST_AUTH));
        strcpy(host_auth,_HOST_AUTH);
    }

    if (autoauthdir == NULL){
	log_error("'autoauthdir' not set.");
    }

    if (strncmp(host_auth,"*",1) != 0){
	if ( (hp = gethostbyname(host_auth)) == NULL ){
	    log_error("'hostauth' not resolved, host not found.");
	}
    }
    check_open( index_path, 0 );
    check_open( utmp_path, 0 );
    check_open( wtmp_path, 0 );
    check_open( deny_identification, 0 );
    check_open( deny_passwd, 0 );
    check_open( deny_right, 0 );
    check_open( userlist, 0 );
    check_open( adminlist, 0 );
    check_open( pluginlist, 0 );
    check_open( pwdlist, 0 );
    check_open( plugin_index, 0 );
    check_open( user_index, 0 );
    check_open( admin_index, 0 );
    check_open( ignoreusers, 0 );
    if (low_serv_port != 0){
	serv_port = low_serv_port;
    }


#ifdef NOSSL
    if (use_ssl == 1){
	use_ssl = 0;
	log_info("SSL disabled: not linked with SSL library.");
    }
#else

    if (use_ssl == 1){
	if ( (check_open( ssl_key, 1 ) != 0)||( check_open( ssl_certificate, 1 ) != 0)){
	    use_ssl == 0;
	    log_info("SSL disabled: key or certificate not found.");
	}
	ssl_init();
    }
#endif
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void reload_config( void ){
	str_array * str_pnt;
	str_array * str_pnt_temp;
        p_hosts_allow    * pnt_hosts_allow;
        p_hosts_allow    * tmp_hosts_allow;
        time_period      * pnt_timeperiods;
        time_period      * tmp_timeperiods;
        plugin_list      * pnt_plugins;
        plugin_list      * tmp_plugins;

    if (low_serv_port == 0){
	serv_port = DEFAULT_PORT;
    } else {
	serv_port = low_serv_port;
    }
    pop3_port = POP3_PORT;
    refresh   = REFRESH_FORM;
    pwdcrypt  = 1;
    logadmins = 0;
    logplugins= 0;
    logusers  = 0;
    logipdeny = 0;
    invertignoreusers = 0;
    invertpluginlist = 0;
    invertadminlist = 0;
    invertuserlist = 0;
    use_ssl = 0;
    permanent_price=0;
    minimum_price=0;
    
    str_pnt = ipallow;
    while (str_pnt != NULL){
	my_free (str_pnt->item);
	str_pnt_temp=str_pnt->next;
	my_free (str_pnt);
	str_pnt = str_pnt_temp;
    } 

    str_pnt = calctty;
    while (str_pnt != NULL){
	my_free (str_pnt->item);
	str_pnt_temp=str_pnt->next;
	my_free (str_pnt);
	str_pnt = str_pnt_temp;
    } 

    pnt_hosts_allow = post_hostsallow;
    while (pnt_hosts_allow != NULL){
	tmp_hosts_allow=pnt_hosts_allow->next;
	my_free (pnt_hosts_allow);
	pnt_hosts_allow = tmp_hosts_allow;
    } 

    pnt_timeperiods = post_timeperiods;
    while (pnt_timeperiods != NULL){
	my_free (pnt_timeperiods->name);
	tmp_timeperiods=pnt_timeperiods->next;
	my_free (pnt_timeperiods);
	pnt_timeperiods = tmp_timeperiods;
    } 

    pnt_plugins = post_plugins;
    while (pnt_plugins != NULL){
	my_free (pnt_plugins->name);
	my_free (pnt_plugins->iostdin);
	my_free (pnt_plugins->proc);
	my_free (pnt_plugins->header);
	tmp_plugins=pnt_plugins->next;
	my_free (pnt_plugins);
	pnt_plugins = tmp_plugins;
    } 

    n_ipallow=0;
    n_calctty=0;
    n_timeperiods=0;
    n_plugins=0;

    my_free(wtmp_path);
    my_free(ignoreusers);
    my_free(utmp_path);
    my_free(index_path);
    my_free(deny_path);
    my_free(host_auth);
    my_free(deny_identification);
    my_free(deny_passwd);
    my_free(deny_right);
    my_free(autoauthdir);
    my_free(userlist);
    my_free(adminlist);
    my_free(pluginlist);
    my_free(pwdlist);
    my_free(plugin_index);
    my_free(user_index);
    my_free(admin_index);
    my_free(ssl_key);
    my_free(ssl_certificate);
    my_free(wtmp_arc_mask);
    my_free(wtmp_arc_expand);

    ipallow = NULL;
    calctty = NULL;
    wtmp_path = NULL;
    utmp_path = NULL;
    index_path = NULL;
    deny_path = NULL;
    host_auth = NULL;
    deny_identification = NULL;
    deny_passwd = NULL;
    deny_right = NULL;
    autoauthdir = NULL;
    userlist = NULL;
    adminlist = NULL;
    pluginlist = NULL;
    ignoreusers = NULL;
    pwdlist = NULL;
    plugin_index = NULL;
    user_index = NULL;
    admin_index = NULL;
    hostsallow = NULL;
    timeperiods = NULL;
    plugins = NULL;
    ssl_key = NULL;
    ssl_certificate = NULL;
    wtmp_arc_mask = NULL;
    wtmp_arc_expand = NULL;

    load_config();
}