/* 
 *  subcfg.c - Subconfiguration parsers in config items.
 *
 *  Copyright (c) 1998  Maxim Chirkov <mc@skyway.ru>  
 */

#include "getstatd.h"

p_hosts_allow    * post_hostsallow;
time_period      * post_timeperiods;
plugin_list      * post_plugins;

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*
 * hostsallow { flag:netaddr:netmask,...}
 * flag:
 * "A" - admins.
 * "P" - plugins.
 * "U" - users.
 * "*" - deny for for users, plugins and admins.
 * "!" - deny connections from this host.
 * ""  - allow for users, plugins and admins.
 */

void prepare_hostsallow(){
	str_array * str_pnt_temp;
	char buff[BUF_SIZE+1];
	str_array * str_pnt;
	char * tok;
	p_hosts_allow * point_arr;
	p_hosts_allow ** cur_next;
	char * input;
	int num=0;

    post_hostsallow = NULL;
    cur_next = &(post_hostsallow);

    str_pnt = hostsallow;
    while (str_pnt != NULL){
	input = str_pnt->item;
	num++;

	if (input[0] == '\0'){
	    snprintf(buff,BUF_SIZE,"IP ACL not defined.");
	    log_error(buff);        
	}
	point_arr = (p_hosts_allow *) my_malloc(sizeof( p_hosts_allow ));
	point_arr->next = NULL;
	*cur_next = point_arr;
	cur_next = &(point_arr->next);
/* Flag */
	if ((tok = strtok(input,":"))== NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect access line: [%s][%d]",input,num);
	    log_error(buff);        
	}

	point_arr->flag_admin=0;
	point_arr->flag_plugin=0;
	point_arr->flag_users=0;
        point_arr->flag_deny=0;
	point_arr->flag_close=0;
        point_arr->flag_all=0;

	if (strpbrk(tok,"Aa") != NULL){
    	    point_arr->flag_admin=1;
	}
	if (strpbrk(tok,"Pp") != NULL){
	    point_arr->flag_plugin=1;
	}
	if (strpbrk(tok,"Uu") != NULL){
	    point_arr->flag_users=1;
	}
	if (strchr(tok,'*') != NULL){
    	    point_arr->flag_deny=1;
	}
	if (strchr(tok,'!') != NULL){
	    point_arr->flag_close=1;
	}
	if (strpbrk(tok,"AaPpUu*!") == NULL){
    	    point_arr->flag_all=1;
	}

/* IP Addr */
	if ((tok=strtok(NULL,":")) == NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect access line: [%s][%d]",input,num);
	    log_error(buff);        
	}
	if ( (point_arr->netaddr = inet_addr(tok)) == -1){
	    snprintf(buff,BUF_SIZE,"Incorrect IP address: [%s][%d]",tok,num);
	    log_error(buff);        
	}
/* IP Mask */
	tok=strtok(NULL,":");
	if ( (tok == NULL)||((point_arr->netmask = inet_addr(tok)) == -1)){
	    point_arr->netmask = inet_addr("255.255.255.255");
	}

	str_pnt=str_pnt->next;
    } 
    str_pnt = hostsallow;
    while (str_pnt != NULL){
	my_free (str_pnt->item);
	str_pnt_temp=str_pnt->next;
	my_free (str_pnt);
	str_pnt = str_pnt_temp;
    } 
    n_hostsallow=0;
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
/* timeperiod { name:base_hour:hours_in_the_period,... } */

void prepare_timeperiod(){
	str_array * str_pnt_temp;
	char buff[BUF_SIZE+1];
	str_array * str_pnt;
	char * tok;
	time_period * point_arr;
	time_period ** cur_next;
	char * input;
	int num=0;


    post_timeperiods = NULL;
    cur_next = &(post_timeperiods);

    str_pnt = timeperiods;
    while (str_pnt != NULL){
	num++;
	input = str_pnt->item;
	if (input[0] == '\0'){
	    log_info("Timeperiods context error.");
	    return;
	}

	point_arr = (time_period *) my_malloc(sizeof( time_period ));
	point_arr->next = NULL;
	*cur_next = point_arr;
	cur_next = &(point_arr->next);
	point_arr->all_time=0;

/* Name */
	if ((tok = strtok(input,":"))== NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect period line: [%s][%d]",input,num);
	    log_error(buff);        
	}

	point_arr->name = (char *) my_malloc(strlen(tok)+1);
        strncpy(point_arr->name, tok, strlen(tok));
        point_arr->name[strlen(tok)] = '\0';

	if (strpbrk(tok," \t\n\r") != NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect period name: [%s][%d]",input,num);
	    log_error(buff);        
	}

/* Base hour */
	if ((tok=strtok(NULL,":")) == NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect base_hour line: [%s][%d]",tok,num);
	    log_error(buff);        
	}
	
#ifdef HAVE_STRTOUL
	errno = 0;
	point_arr->base_hour = (unsigned int) strtoul (tok, NULL, 0);
	if ((errno)||(point_arr->base_hour >24)){
	    snprintf(buff,BUF_SIZE,"Incorrect base_hour line: [%s][%d]",tok,num);
	    log_error(buff);        
	}
#else
	if ((sscanf(point_arr->base_hour,"%u", tok) == EOF)||(point_arr->base_hour >24)){
	    snprintf(buff,BUF_SIZE,"Incorrect base_hour line: [%s][%d]",tok,num);
	    log_error(buff);        
	}
#endif
	
/* Period */
	if ((tok=strtok(NULL,":")) == NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect base_period line: [%s][%d]",tok,num);
	    log_error(buff);        
	}

#ifdef HAVE_STRTOUL
	errno = 0;
	point_arr->period = (unsigned int) strtoul (tok, NULL, 0);
	if ((errno)||(point_arr->period >24)){
	    snprintf(buff,BUF_SIZE,"Incorrect base_period line: [%s][%d]",tok,num);
	    log_error(buff);        
	}
#else
	if ((sscanf(point_arr->period,"%u", tok) == EOF)||(point_arr->period >24)){
	    snprintf(buff,BUF_SIZE,"Incorrect base_period line: [%s][%d]",tok,num);
	    log_error(buff);        
	}
#endif

/* Price */
	if ((tok=strtok(NULL,":")) == NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect price_for_minute line: [%s][%d]",tok,num);
	    log_error(buff);        
	}

#ifdef HAVE_STRTOD
	errno = 0;
	point_arr->price = (float) strtod (tok, NULL);
	if (errno){
	    snprintf(buff,BUF_SIZE,"Incorrect price_for_minute line: [%s][%d]",tok,num);
	    log_error(buff);        
	}
#else
	if (sscanf(point_arr->price,"%f", tok) == EOF){
	    snprintf(buff,BUF_SIZE,"Incorrect price_for_minute line: [%s][%d]",tok,num);
	    log_error(buff);        
	}
#endif
	str_pnt=str_pnt->next;
    } 

    str_pnt = timeperiods;
    while (str_pnt != NULL){
	my_free (str_pnt->item);
	str_pnt_temp=str_pnt->next;
	my_free (str_pnt);
	str_pnt = str_pnt_temp;
    } 

}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
/* plugins { name:stdin:proc:header } */
/*  char * name;
    char * iostdin;
    char * proc;
    char * header;
    struct _PLUGIN_LIST_ * next;
*/
void prepare_plugins(){
	str_array * str_pnt_temp;
	char buff[BUF_SIZE+1];
	str_array * str_pnt;
	char * tok;
	plugin_list * point_arr;
	plugin_list ** cur_next;
	char * input;
	int num=0;


    post_plugins = NULL;
    cur_next = &(post_plugins);

    str_pnt = plugins;
    while (str_pnt != NULL){
	num++;
	input = str_pnt->item;
	if (input[0] == '\0'){
	    log_info("Plugin context error.");
	    return;
	}

	point_arr = (plugin_list *) my_malloc(sizeof( plugin_list ));
	point_arr->next = NULL;
	*cur_next = point_arr;
	cur_next = &(point_arr->next);

/* Name */
	if ((tok = strtok(input,":"))== NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect plugin line: [%s][%d]",input,num);
	    log_error(buff);        
	}

	point_arr->name = (char *) my_malloc(strlen(tok)+1);
        strncpy(point_arr->name, tok, strlen(tok));
        point_arr->name[strlen(tok)] = '\0';

	if (strpbrk(tok," \t\n\r@") != NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect plugin name: [%s][%d]",input,num);
	    log_error(buff);        
	}

/* iostdin */
	if ((tok = strtok(NULL,":"))== NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect iostdin in plugin line: [%s][%d]",input,num);
	    log_error(buff);        
	}

	point_arr->iostdin = (char *) my_malloc(strlen(tok)+1);
        strncpy(point_arr->iostdin, tok, strlen(tok));
        point_arr->iostdin[strlen(tok)] = '\0';

/* proc */
	if ((tok = strtok(NULL,":"))== NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect proc in plugin line: [%s][%d]",input,num);
	    log_error(buff);        
	}

	point_arr->proc = (char *) my_malloc(strlen(tok)+1);
        strncpy(point_arr->proc, tok, strlen(tok));
        point_arr->proc[strlen(tok)] = '\0';

/* header */
	if ((tok = strtok(NULL,":"))== NULL){
	    snprintf(buff,BUF_SIZE,"Incorrect header in plugin line: [%s][%d]",input,num);
	    log_error(buff);        
	}

	point_arr->header = (char *) my_malloc(strlen(tok)+1);
        strncpy(point_arr->header, tok, strlen(tok));
        point_arr->header[strlen(tok)] = '\0';


	str_pnt=str_pnt->next;
    } 

    str_pnt = plugins;
    while (str_pnt != NULL){
	my_free (str_pnt->item);
	str_pnt_temp=str_pnt->next;
	my_free (str_pnt);
	str_pnt = str_pnt_temp;
    } 
}